/*
 * Copyright © 2020 Keith Packard <keithp@keithp.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#pragma once

/*
 *	Label	Port	Func	PWM
 *	D0	PB0
 *	D1	PB1
 *	D2	PB2
 *	D3	PB3		OC0A
 *	D4	PB4	SS	OC0B
 *	D5	PB5	MOSI
 *	D6	PB6	MISO	OC3A
 *	D7	PB7	SCK	OC3B
 *
 *	D8	PD0	RX0
 *	D9	PD1	TX0
 *	D10	PD2	RX1
 *	D11	PD3	TX1
 *	D12	PD4		OC1B
 *	D13	PD5		OC1A
 *	D14	PD6		OC2B
 *	D15	PD7		OC2A
 *
 *	D16	PC0	SCL
 *	D17	PC1	SDA
 *	D18	PC2	TCK
 *	D19	PC3	TMS
 *	D20	PC4	TDO
 *	D21	PC5	TDI
 *	D22	PC6
 *	D23	PC7
 *
 *	A0	PA0
 *	A1	PA1
 *	A2	PA2
 *	A3	PA3
 *	A4	PA4
 *	A5	PA5
 *	A6	PA6
 *	A7	PA7
 */
